"use strict";

// prettier-ignore
require('@babel/register')({
  // @x-shipit-disable
  ignore: [/node_modules\/(?!@adeira)/],
  // @x-shipit-disable
  rootMode: 'upward' // @x-shipit-disable

}); // @x-shipit-disable


var template = require('@babel/template').default;

var t = require('@babel/types');

var murmurHash = require('@adeira/murmur-hash').default;

var _require = require('@adeira/sx-tailwind'),
    tailwindStyles = _require.tailwindStyles;

module.exports = function sxTailwindBabelPlugin() {
  var stylesCollector = [];
  var tailwindCallee = '';
  var sxtCallee = '';
  var stylesVarName = 'styles';
  return {
    name: 'sx-tailwind-babel-transform',
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        if (path.node.source.value === '@adeira/sx-tailwind') {
          path.node.specifiers.forEach(function (_ref) {
            var imported = _ref.imported,
                local = _ref.local;

            if (imported.name === 'tailwind') {
              tailwindCallee = local.name;
            } else if (imported.name === 'sxt') {
              sxtCallee = local.name;
            }
          });
          var sxImport = template.ast("import * as sx from '@adeira/sx'");
          path.replaceWith(sxImport);
        }
      },
      JSXAttribute: function JSXAttribute(path) {
        if (path.node.name.type !== 'JSXIdentifier' || path.node.name.name !== 'className' || path.node.value.type !== 'JSXExpressionContainer' || path.node.value.expression.type !== 'CallExpression' || path.node.value.expression.callee.type !== 'Identifier') {
          return;
        }

        if (path.node.value.expression.callee.name === sxtCallee) {
          path.node.value.expression.arguments.forEach(function (a) {
            return stylesCollector.push(a.value);
          });
          path.node.value.expression.callee.name = stylesVarName;
        } else if (path.node.value.expression.callee.name === tailwindCallee) {
          var styles = [];
          path.node.value.expression.arguments[0].value.split(' ').forEach(function (style) {
            stylesCollector.push(style);
            styles.push(style);
          });
          var attribute = t.jsxAttribute(t.jsxIdentifier('className'), t.jsxExpressionContainer(t.callExpression(t.identifier(stylesVarName), styles.map(function (style) {
            return t.stringLiteral(style);
          }))));
          path.replaceWith(attribute);
        }
      },
      Program: {
        enter: function enter(path, state) {
          stylesCollector = [];
          var filename = state.file.opts.filename.split('/').slice(-2).join('/');
          stylesVarName = "__styles_".concat(murmurHash(filename));
        },
        exit: function exit(path) {
          var declarations = Object.fromEntries(stylesCollector.map(function (style) {
            return [style, tailwindStyles[style]];
          }));

          if (Object.keys(declarations).length === 0) {
            return;
          }

          path.node.body.push(template.ast("const ".concat(stylesVarName, " = sx.create(").concat(JSON.stringify(declarations), ")"))); // is there SX imported twice?

          var sxImports = path.node.body.reduce(function (a, _ref2, index) {
            var type = _ref2.type,
                source = _ref2.source,
                specifiers = _ref2.specifiers;

            if (type === 'ImportDeclaration' && source.value === '@adeira/sx' && Array.isArray(specifiers) && specifiers[0].type === 'ImportNamespaceSpecifier') {
              a.push(index);
            }

            return a;
          }, []);

          if (sxImports.length > 1) {
            delete path.node.body[sxImports[0]];
          }
        }
      }
    }
  };
};